@@ -1,5 +1,5 @@
 package Mojo::DOM::Role::Restrict;
-use strict; use warnings; our $VERSION = 0.02;
+use strict; use warnings; our $VERSION = 0.03;
 use Mojo::Base -role;
 use Mojo::Util qw(xml_escape);
 
@@ -23,6 +23,10 @@ sub restrict_spec {
 	${$_[0]}->{restrict_spec};
 }
 
+sub valid { _valid($_[0]->tree, $_[0]->restrict_spec($_[1])) }
+
+sub restrict { _restrict($_[0]->tree, $_[0]->restrict_spec($_[1])) && $_[0] }
+
 # copy, paste and edit via Mojo::DOM::HTML::_render
 
 my %EMPTY = map { $_ => 1 } qw(area base br col embed hr img input keygen link menuitem meta param source track wbr);
@@ -108,9 +112,48 @@ sub _valid_attribute {
 			: 0;
 }
 
+sub _valid {
+	my ($tree, $spec) = @_;
+	if ($tree->[0] eq 'tag') {
+		my ($tag, $attrs) = _valid_tag($spec, $tree->[1], {%{$tree->[2]}});
+		return 0 unless $tag;
+		_valid_attribute($spec, $tag, $_, $attrs->{$_}) or return 0
+			for (sort keys %{$attrs});
+		if ($tree->[4]) {
+			_valid($_, $spec) or return 0 for ( @$tree[4 .. $#$tree]  );
+		}
+	} elsif ($tree->[0] eq 'root') {
+		_valid($_, $spec) or return 0 for ( @$tree[1 .. $#$tree] );
+	}
+	return 1;
+}
+
+sub _restrict {
+	my ($tree, $spec) = @_;
+	if ($tree->[0] eq 'tag') {
+		my ($tag, $attrs) = _valid_tag($spec, $tree->[1], $tree->[2]);
+		return 0 unless $tag;
+		$tree->[1] = $tag;
+		for (sort keys %{$attrs}) {
+			my ($key, $value) = _valid_attribute($spec, $tag, $_, delete $attrs->{$_});
+			$attrs->{$key} = $value if $key;
+		}
+		if ($tree->[4]) {
+			my $i = 4;
+			_restrict($_, $spec) ? $i++ : splice(@{$tree}, $i, 1) 
+				for ( @$tree[$i .. $#$tree]  );
+		}
+	} elsif ($tree->[0] eq 'root') {
+		my $i = 1;
+		_restrict($_, $spec) ? $i++ : splice(@{$tree}, $i, 1)  
+			for ( @$tree[$i .. $#$tree] );
+	}
+	return 1;
+}
+
 1;
 
-# TODO valid, restrict
+# TODO diff
 
 __END__
 
@@ -120,7 +163,7 @@ Mojo::DOM::Role::Restrict - Restrict tags and attributes
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
@@ -193,10 +236,23 @@ Version 0.02
 	# <html><head><script>...</script></head><body><p onclick="not-allow">Restrict <span class="not-okay">HTML</span></p></body></html>
 	$dom->to_string;
 
+	# check whether the spec is valid
+	$dom->valid; # 0
+
+	# apply spec changess to the Mojo::DOM object
+	$dom->restrict;
+
+	# re-check whether the spec is valid
+	$dom->valid; # 1
+
+	# render using original render function (Mojo::DOM::HTML::render)
+	# <html><head><script>...</script></head><body><p onclick="not-allow">Restrict <span class="not-okay">HTML</span></p></body></html>
+	$dom->to_string(1);
+
 	$dom->parse(q|<p class="okay" data-unknown="abc" id="prefixed-allow" onclick="not-allow">Restrict <span class="not-okay">HTML</span></p>|);
 
 	# <p onclick="not-allow">Restrict <span class="not-okay">HTML</span></p>
-	$dom->to_string, 
+	$dom->to_string;
 
 
 =head1 SUBROUTINES/METHODS
@@ -211,6 +267,85 @@ Retrieve/Set the specification used to restrict the HTML.
 
 =cut
 
+=head2 valid
+
+Validate the current DOM against the specification. Returns true(1) if valud returns false(0) if invalid.
+
+	my $html = q|<html><head><script>...</script></head><body><p class="okay" id="allow" onclick="not-allow">Restrict <span class="not-okay">HTML</span></p></body></html>|;
+
+	my $spec = {
+		html => 1,
+		head => 1,
+		script => 1,
+		body => 1,
+		p => 1,
+		span => 1
+	};
+
+	my $dom = Mojo::DOM->with_roles('+Restrict')->new($html, $spec); 
+
+	$dom->valid; # 1;
+
+	$spec = {
+		html => 1,
+		head => 1,
+		script => 1,
+		body => 1,
+		p => 1,
+		span => 0
+	};
+
+	$dom->valid($spec); # 0;
+
+=cut
+
+=head2 restrict 
+
+Restrict the current DOM against the specification, after calling restrict the specification changes applied become irreversible.
+
+	my $html = q|<html><head><script>...</script></head><body><p class="okay" id="allow" onclick="not-allow">Restrict <span class="not-okay">HTML</span></p></body></html>|;
+
+	my $spec = {
+		script => 0, # no script tags
+		'*' => { # allow all tags
+			'*' => 1, # allow all attributes
+			onclick => sub { 0 }, # disable onclick attributes
+			id => sub { return @_ }, # enable id attributes
+			class => sub { # allow only 1 class 'okay'
+				my ($attr, $val) = @_;
+				my $match = $val =~ m/^okay$/;
+				return $match ? ($attr, $val) : 0;
+			}
+		},
+		span => {
+			validate_tag => sub { # replace span tags with b tags
+				return ('b', $_[1]);
+			}
+		},
+		p => {
+			validate_tag => sub {
+				$_[1]->{id} = "prefixed-" . $_[1]->{id}; # prefix all p tag IDs
+				$_[1]->{'data-unknown'} = 'abc';  # extend all p tags with a data-unknown attribute
+				return @_;
+			}
+		},
+	};
+	
+	$dom->parse($html, $spec);
+
+	# render without spec validation
+	# <html><head><script>...</script></head><body><p class="okay" id="allow" onclick="not-allow">Restrict <span class="not-okay">HTML</span></p></body></html>
+	$dom->to_string(1);
+	
+	# restrict the DOM
+	$dom->restrict;
+	
+	# render without spec validation
+	# <html><head></head><body><p class="okay" data-unknown="abc" id="prefixed-allow">Restrict <b>HTML</b></p></body></html>
+	$dom->to_string(1);
+
+=cut
+
 =head1 AUTHOR
 
 LNATION, C<< <email at lnation.org> >>
